/* M. Fuss 11.11.2007
 * Stdio demo
 *
 * 
 */

#include "defines.h"

#include <ctype.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <avr/io.h>
#include <avr/pgmspace.h>


#include <util/delay_basic.h>

#include "uart.h"

#define SET(PORT,BIT)	PORT |= _BV(BIT)
#define RES(PORT,BIT)	PORT ^= _BV(BIT)


#define RW			PC1
#define RS			PC0
#define EN			PC7
#define RES_LCD		PC6

#define SPALTEN_MAX 20
#define ZEILEN_MAX	4


FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);


/*
 * Do all the startup-time peripheral initializations.
 */

static void
ioinit(void)
{
  uart_init();
  
  // PORT D Bit 6,7 is output	

	DDRD |= _BV(PD7);
	DDRD |= _BV(PD6);
	
	// Port B = output	DATA f. LCD

	DDRB = 0xff;
	
	// Por C Bit 0,1 ,6 ,7  is output controllines f. LCD

	DDRC |= _BV(RW);
	DDRC |= _BV(RS);
	DDRC |= _BV(EN);
	DDRC |= _BV(RES_LCD);	
  
}






static void
delay_1s(void)
{
  uint8_t i;

  for (i = 0; i < 100; i++)
    _delay_loop_2(10);
}







void beep(unsigned count)
{
	while (count != 0) 
	{

		SET(PORTD,PD7);
		_delay_loop_2(20000);
		RES(PORTD,PD7);
		_delay_loop_2(20000);
		count--;
	}

}

// -------------------------------
// Byte auf Display ausgeben
// comand = 1 bytewert ist Komando 
// -------------------------------
void write_LCD(char byte, char comand)
{

static char lcd_busy = 1;

	
	DDRB = 0x00;		// port B Input
	
	SET(PORTC,RW);
	while(lcd_busy)
	{
		SET(PORTC,EN);
		if(!bit_is_set(PINB,PD7))
			lcd_busy = 0;
		RES(PORTC,EN);
		_delay_loop_2(200);
		
	}
	DDRB = 0xff;		// port B wieder Output

	RES(PORTC,RW);
	lcd_busy = 1;

	//_delay_loop_2(2000);

	if(!comand) SET(PORTC,RS);
	PORTB = byte;
	SET(PORTC,EN);
		_delay_loop_2(200);
	RES(PORTC,EN);
	if(!comand) RES(PORTC,RS);

}



void init_LCD(void)
{
int i;
		
	SET(PORTC,RES_LCD);	// LCD Reset off

	write_LCD(0x30,1);	// 8Bit BUS RE = 0
	write_LCD(0x06,1);	// Cursor autoincrement
	write_LCD(0x36,1);	// 8 Bit RE=1 Blink =1
	write_LCD(0x09,1);	// 4 zeilen

	// all 16 icons not visible

	write_LCD(0x40,1);	// Icon Ram Adress
	for(i=0; i<16; i++)	
			write_LCD(0x00,0); 

	write_LCD(0x30,1);	// 8 bit Bus RE = 0
	write_LCD(0x0C,1);	// Displ. ein cursor aus
	
	
	write_LCD(0x01,1);	// clear display

	
}


// ***********************************************************************
// Ausgabe einer Zeichenkette positioniert auf dem LCD-Modul
// bergabe: lcd_x : Spalte (0...SPALTEN_MAX-1)
//           lcd_y : Zeile  (0...ZEILEN_MAX-1)
//           lcd_zeichen : Adresse der auszugebenden format.  Zeichenkette
//           clr_line    : Lschen bis Zeilenende
//                         1 - Lschen
//                         0 - kein Lschen
// ***********************************************************************

void printf_LCD(char lcd_x, char lcd_y, char *lcd_zeichen, char clr_line) 
{

char lcd_i;
char lcd_offset = 0;

  if (lcd_x > (SPALTEN_MAX - 1)) lcd_x = 0;
  if (lcd_y > (ZEILEN_MAX  - 1)) lcd_y = 0;

  switch (lcd_y) 
  {
    case 0:  lcd_offset = 0x00; break;  // Zeile 1
    case 1:  lcd_offset = 0x20; break;  // Zeile 2 
    case 2:  lcd_offset = 0x40; break;  // Zeile 3
    case 3:  lcd_offset = 0x60; break;  // Zeile 4 
    default: lcd_offset = 0x00;         // Zeile 1
  }


	lcd_x += lcd_offset;    // Kursorposition setzen, hierzu muss Bit 7 immer gesetzt sein 
	lcd_x |= _BV(7);        // siehe Datenblatt LCD DD-RAM Adresse set	
	
  	write_LCD(lcd_x,1); 

	lcd_x ^= _BV(7); 		// original x koordinate wieder herstellen, Bit 7 lschen  
	lcd_x -= lcd_offset;		// lcd_offset wieder abziehen
  	

  // Ausgabe der Zeichenkette ab der Kursorposition

  lcd_offset = strlen(lcd_zeichen);     // Lnge der Zeichenkette

  if (lcd_offset > SPALTEN_MAX) lcd_offset = SPALTEN_MAX;

  for(lcd_i = lcd_offset ; lcd_i ; lcd_i--) 
  {
    	write_LCD(*lcd_zeichen,0);
    	lcd_zeichen++;
  }


  if (clr_line) 
  {
     // Lschen bis Zeilenende
     for(lcd_i = SPALTEN_MAX - lcd_offset - lcd_x ; lcd_i ; lcd_i--)
	 write_LCD(' ',0);
  }

}


int
main(void)
{
  int anzahl=0;

  char buf[20];

  ioinit();

  stdout = stdin = &uart_str;

	init_LCD();
	
	printf_LCD(0,0,"Martin Fuss",1); 
	printf_LCD(0,1,"uC  = ATmega32",1);
	printf_LCD(0,2,"LCD = EA DIP204-6",1);  
		
	
  for (;;)
  {

  		
   		   	
		printf("Enter command: ");

		// read until RETURN or until end of buf is reached

      	if (fgets(buf, sizeof buf - 1, stdin) == NULL)
			break;
      	if (buf[0] == 'q')
			break;
	
      	switch (buf[0])
		{
		default:
	  		 	printf("Unknown command: %s\n", buf);
				//printf_LCD(0,1,"Theo",1); 
				printf_LCD(0,3,buf,1); 
		break;

	
		// example: command over RS232 ->  b10     will beep ten times
		case 'b':
				
				anzahl = atoi(&buf[1]);
				beep(anzahl);
	  			printf("case piepsen = %s\n", buf);


	    break;

		case '1':
				
				SET(PORTD,PD6);  		
	       		printf("case 1 = %s\n", buf);
	    break;

		case '2':
				
				RES(PORTD,PD6); 
	  			printf("case 3 = %s\n", buf);
	    	
	  	break;
		}

    }

	printf("\nProgram is at an end");
	 
  return 0;
}
